User = {
  
    init: function(){
		this.moduleName = 'expressoCalendar';
		this.me = DataLayer.dispatch( "me" );
		
		var objTime = DataLayer.dispatch( "timezones" );
		this.timezones = objTime.timezones;
		this.isDaylightSaving = objTime.isDaylightSaving;
		this.load();      
    },
  
    load: function(){

		var defaultPreferences = { dateFormat: "dd/MM/yyyy",
			hourFormat: "HH:mm",
			defaultCalView: "month",
			defaultDuration: 30,
			backgroundColor: "36C",
			borderColor: "36C",
			fontColor: "fff",
			timezone: 'America/Sao_Paulo',
			weekStart: 'SUN'
		  };

		var pref = DataLayer.get( "modulePreference:detail", ['and',[ "=", "user", this.me.id ], [ "=", "module", this.moduleName ]] );

		this.preferences = DataLayer.merge( defaultPreferences, pref.values || {} );
		this.preferenceIds = pref.ids;
    }
}

UI = {
	dialogs: {
		addEvent: null,
		importCalendar: null
	}
}

DataLayer.codec( "calendarSignature", "calendar", {
  
	decoder: function(){},
      
	encoder: function( signatures ){

		return $.map( signatures, function( signature ){
			return { events: function( start, end, callback ){
				var viewKey = start + ':' + end;
				if( Calendar.currentViewKey !== viewKey ){
				    Calendar.currentViewKey = viewKey;
				    Calendar.currentView = DataLayer.get( 'schedulable:calendar', { start: start, end: end } );
				}
				var view = Calendar.currentView[ signature.calendar.id ];
				callback( view && !view.hidden ? view : [] );
			   },

			   backgroundColor: '#' + signature.backgroundColor || User.preferences.backgroundColor,
			   borderColor: '#' + signature.borderColor || User.preferences.borderColor,
			   textColor: '#' + signature.fontColor || User.preferences.fontColor,
			   className: 'calendar-' + signature.calendar.id,
			   editable:  signature.isOwner }
	       });
      }

      
});

Calendar = {
  
	load: function(){
		this.lastView = 0;
		this.signatures  = DataLayer.get("calendarSignature", {filter: ['=', 'user', User.me.id], criteria: {deepness: 2}});
	  
		var prevSources = this.sources;

		this.sources = DataLayer.encode( "calendarSignature:calendar", this.signatures );

		if( prevSources ){     
			var newSources = DataLayer.diff( { 'diff': prevSources }, { 'diff': this.sources } ).diff;

			for( var i = 0; i < newSources.length; i++ )
				$('#calendar').fullCalendar( 'addEventSource', newSources[i] );

			var removeSources = DataLayer.diff( { 'diff': newSources }, { 'diff': prevSources } ).diff;

			for( var i = 0; i < removeSources.length; i++ )
				$('#calendar').fullCalendar( 'removeEventSource', removeSources[i] );
		}

		this.calendarIds = [], this.signatureOf = {}, this.calendars = [], this.calendarOf= {};

		for( var i = 0; i < this.signatures.length; i++ ){
			this.signatureOf[ this.calendarIds[i] = ( this.calendars[ this.calendars.length ] = this.calendarOf[ this.signatures[i].id ] = this.signatures[i].calendar ).id ] = this.signatures[i];
		}

		delete Calendar.currentViewKey;
	},

	rerenderView: function(force){

		if(force){
			delete Calendar.currentViewKey;
			$('#calendar').fullCalendar( 'refetchEvents' );
		}

		var calendarNotSelected = getSelectedCalendars( true );
		for(var i = 0; i < calendarNotSelected.length; i++)
			if(!!Calendar.currentView[ calendarNotSelected[i] ])
				Calendar.currentView[ calendarNotSelected[i] ].hidden = true;

		$('#calendar').fullCalendar( 'refetchEvents' );	
	}
}

Alarms = {
	load: function(){
		var eventsDay = DataLayer.get('alarm:schedulable',['=', 'date', new Date().toString("yyyy-MM-dd")]);
		for(var i = 0; i < eventsDay.length; i++){
			this.addAlarm( eventsDay[i] );
		}
	},
	
	addAlarm: function( eventDay ){
		DataLayer.task( parseInt(eventDay.sendTime) , function( timestamp ){
			DataLayer.render('templates/alarm.ejs',{event: eventDay}, function( html ){
				var dlgAlarm = jQuery('#div-alarm').html( html ).find('.alarm-event.active');
				dlgAlarm.dialog({resizable: false, modal:true, width:400, minHeight: 100,position: 'center', close:function(){$(this).dialog('destroy');$(this).dialog('open');}});
			});
		});
	}
}

DataLayer.codec( "modulePreference", "detail", {
	decoder: function( evtObj ){

	if( notArray = $.type(evtObj) !== "array" )
		evtObj = [ evtObj ];
		
		var res = $.map(evtObj, function( form ){

			var returns = [];

			for (var name in form)
				returns[ returns.length ] = {
					name: name,
					user: User.me.id,
					value: form[name],
					module: User.moduleName,
					id: User.preferenceIds[ name ] || undefined
				};

			return [returns];
		});
	
	return notArray ? res[0] : res;
	},

    encoder: function( evtObj ){
		var val = {}, id = {};
		for (var i in evtObj){
			
			if( evtObj[i].value && evtObj[i].id )
			{
			    val[evtObj[i].name] = evtObj[i].value;
			    id[evtObj[i].name] = evtObj[i].id;
			}
		}
		return {values: val,ids: id};
	
	}
});

User.init();
Calendar.load();

/* Todo ser implementado funcionalidades ajax no calback de importacao de eventos */
isValidCalback = false;

/*Todo Melhorias sero feitas na API*/
//DataLayer.poll( "schedulable" );

DataLayer.listen( "schedulable", function( status, updateData){
  
    if( status === 'serverclient' ){
		DataLayer.commit( false, false, function(){	
			if(!!$('#calendar').fullCalendar)
				Calendar.rerenderView(true);
			//Atualiza a listagem de eventos apenas se estiver na aba de listagem
			if((typeof($tabs) != "undefined") && $tabs.tabs('option' ,'selected') != 0)
				pageselectCallback($('.events-list-win.active [name=keyword]').val(), 0);
			
		});
    }
});

DataLayer.listen( "calendarToSchedulable", function( status, updateData){
  
    if( status === 'client' ){
		if(updateData && isValidCalback){
			isValidCalback = false;
			$.Zebra_Dialog('Evento(s) importados com sucesso!', {
				'overlay_opacity': '0.5',
				'onClose':  function(clicked) {				
					Calendar.rerenderView();
				}
			});
		}else if(updateData == false){
			$.Zebra_Dialog('Erro ao importar evento(s)!', {
				'overlay_opacity': '0.5'	
			});
		}
    };	
});

DataLayer.listen( "calendarSignature", function( status ){

    switch( status ){
		case 'serverclient':  DataLayer.commit( false, false, function( received ){
			delete Calendar.currentViewKey;
			Calendar.load();
			refresh_calendars();
		});
		break;
		case 'client':  
		break;
    }
});

DataLayer.listen( "modulePreference", function( status ){

    switch( status ){
		case 'serverclient':  DataLayer.commit( false, false, function(){
			User.load();
		});
		break;
		case 'client':  
		break;
    }

});

DataLayer.listen( "alarm", function( created, updated, deleted ){

});

/*DataLayer.listen( "calendar", function( status, updateData ){

    if (updateData === false)
	switch( status ){
		case 'serverclient':  DataLayer.commit( false, false, function(){
	    
			Calendar.load();
			refresh_calendars();
	    
		});
		break;
		case 'client':  
		break;
    }
});*/

DataLayer.codec( "calendarSignature", "configure", {
	decoder: function( evtObj ){
		Encoder.EncodeType = "entity";
		
		if( notArray = $.type(evtObj) !== "array" )
			evtObj = [ evtObj ];
		
		var res = $.map(evtObj, function( form ){
			return{
				id: form.signature,
				user: User.me.id,	
				calendar: {
					id: Calendar.calendarOf[form.signature].id,
					name: Encoder.htmlEncode(form.name),
					description: Encoder.htmlEncode(form.description),
					timezone: form.timezone,
					defaultDuration: form.duration != "" ? form.duration : 30,
					location: form.location
				},
				isOwner: 1,
				fontColor: Encoder.htmlEncode(form.fontColor.substring(1)),
				backgroundColor: Encoder.htmlEncode(form.backgroundColor.substring(1)),
				borderColor: Encoder.htmlEncode(form.borderColor.substring(1)),
				msgAdd: Encoder.htmlEncode(form.msgAdd),
				msgCancel: Encoder.htmlEncode(form.msgCancel),
				msgUpdate: Encoder.htmlEncode(form.msgUpdate),
				msgReply: Encoder.htmlEncode(form.msgReply),
				msgAlarm: Encoder.htmlEncode(form.msgAlarm),
				calendarSignatureAlarms: $.map( form.alarmTime || [], function( alarmTime, i ){
					return (!!form.alarmId[i] ?
							{type: form.alarmType[i],	unit: form.alarmUnit[i], time: form.alarmTime[i], id: form.alarmId[i]} :
							{type: form.alarmType[i],	unit: form.alarmUnit[i], time: form.alarmTime[i]});
				})
			};
		
		});
	
	return notArray ? res[0] : res;
	},

    encoder: function( evtObj ){}
});

DataLayer.codec( "alarm", "schedulable", {

    decoder: function( evtObj ){},
	
	encoder: function (evtObjt){
	
			if(notArray = typeof(evtObjt) == 'array' )
				evtObjt = [evtObjt];
			
			var res = $.map(evtObjt, function( objEvent ){	

				var time = parseInt(objEvent.schedulable.startTime);
											
				return{
					id: objEvent.id,
					event_start: new Date( time ).setTimezoneOffset(Timezone.timezones[objEvent.schedulable.timezone]).toString( User.preferences.dateFormat),
					sendTime: objEvent.sendTime,
					schedulable: {
						startTime: dateCalendar.formatDate(Timezone.setDateTimezone(new Date(time), objEvent.schedulable.timezone, true), User.preferences.hourFormat),
						id: objEvent.schedulable.id,
						summary: objEvent.schedulable.summary,
						time: objEvent.time,	
						unit: dateCalendar.timeunit[objEvent.unit.toLowerCase()] + ( parseInt(objEvent.time) > 1 ? 's' : '' )
					}
				}
			});
		return notArray ? res[0] : res;
	}
});

DataLayer.codec( "schedulable", "preview", {

    decoder: function( evtObj ){
		Encoder.EncodeType = "entity";
	
		if( notArray = $.type(evtObj) !== "array" )
			evtObj = [ evtObj ];

		var pref = User.preferences;

		var res = $.map(evtObj, function( form ){
			
			return {
				id: form.idEvent,
				calendar: form.calendar,
				participants : [{
					user: User.me.id,
					status: form.status,
					id: form.meId,
					isOrganizer: 0,
					alarms: typeof(form.alarmTime) != 'undefined' ? 
						$.map( form.alarmTime || [], function( alarmTime, i ){

							if( alarmTime === "" )
								return( null );

							return !!form.alarmId[i] ?
								{ type: form.alarmType[i], unit: form.alarmUnit[i], time: form.alarmTime[i], id: form.alarmId[i] } : 
								{ type: form.alarmType[i],unit: form.alarmUnit[i], time: form.alarmTime[i] };
						}) : []
					}]
			}
		});
		
		return notArray ? res[0] : res;
    },

    encoder: function( evtObj ){

	var notArray = false;
      
	if( notArray = $.type(evtObj) !== "array" )
	    evtObj = [ evtObj ];

	var statusParticipants = {},
	
	statusLabels = [ '', 'accepted', 'tentative', 'cancelled', 'unanswered', 'delegated' ];
	
	for( var i = 0; i < statusLabels.length; i++ )
		statusParticipants[ statusLabels[i] ] = 0;

	var isOrganizer = 1;
	
	var res = $.map(evtObj, function( objEvent ){			
		    
			var participantInfo = {}, me = DataLayer.copy(User.me);
						
		    return {
			    "class": objEvent["class"],
			    id: objEvent.id,
			    location: objEvent.location,
			    category: objEvent.category,
			    calendars: Calendar.calendars,
			    calendar: objEvent.calendar,
			    summary: objEvent.summary,
			    description: objEvent.description,
			    timezone: objEvent.timezone,
			    timezones: Timezone.timezones,
			    startDate: new Date( parseInt(objEvent.startTime) ).setTimezoneOffset( Timezone.timezone( objEvent.timezone ) ).toString( User.preferences.dateFormat ),
			    startHour: dateCalendar.formatDate(Timezone.setDateTimezone(new Date( parseInt(objEvent.startTime)), objEvent.timezone, objEvent.calendar), User.preferences.hourFormat),
			    endDate: new Date( parseInt(objEvent.endTime) - (!!parseInt(objEvent.allDay) ? 86400000 : 0)  ).setTimezoneOffset( Timezone.timezone( objEvent.timezone ) ).toString( User.preferences.dateFormat ),
			    endHour: dateCalendar.formatDate(Timezone.setDateTimezone(new Date(parseInt(objEvent.endTime)), objEvent.timezone, objEvent.calendar), User.preferences.hourFormat),
			    allDay: !!parseInt( objEvent.allDay ),
	  // 				attachments: this.decode( "attachment:detail", form ),
			    attendee: $.map(objEvent.participants || [], function( participant, i ){						
					
					var attend = DataLayer.get('participant', participant);
					attend.user = DataLayer.get('user', attend.user);
					
					statusParticipants[ statusLabels[attend.status] ]++;  	
					
					if( attend.user.id ===  me.id ){
					    participantInfo.user = {
							status : attend.status,
							alarms : $.map(attend.alarms || [], function( alarm ){
								return DataLayer.get('alarm', alarm);
							})
						};
						me.id = attend.id;
                                                return(null);
					};
					
					var person = {
						id: attend.id,
						name: attend.user.name != 'false' ? attend.user.name : '',
						mail: attend.user.mail,
						isExternal: attend.isExternal
					};
					
					if( !!parseInt(attend.isOrganizer )){
						isOrganizer = ((attend.user.id == User.me.id) || (!!!attend.user)) ? 1 : 0;
					    participantInfo.organizer = DataLayer.copy(person);
						return(null);
					};					
					
					return (person);
				}),
			    organizer: participantInfo.organizer || me,
			    alarms: !!participantInfo.user ?  participantInfo.user.alarms : [],
			    status: !!participantInfo.user ? participantInfo.user.status :  1,
			    me: me,
				isOrganizer: isOrganizer,
				statusParticipants: statusParticipants
			};
		});

	return notArray ? res[0] : res;
    }
});



DataLayer.codec( "schedulable", "detail", {

    decoder: function( evtObj ){
		
	Encoder.EncodeType = "entity";
	
	if( notArray = $.type(evtObj) !== "array" )
		evtObj = [ evtObj ];

	var pref = User.preferences;

	var res = $.map(evtObj, function( form ){
    
		    var tzId =  form.timezone || Calendar.signatureOf[form.calendar].calendar.timezone || User.preferences.timezone,

		    formatString = ( typeof form.allDay !== "undefined" && !!form.allDay )? pref.dateFormat+" HH:mm" : pref.dateFormat + " " + pref.hourFormat,
			
		    participants = []; 
		    return {
			      "class": form["class"],
				  startTime: Timezone.getDateUTC(Date.parseExact(form.startDate + (!!form.allDay ? " 00:00": " "+$.trim(form.startHour)) , formatString ), tzId).getTime(),
				  endTime:  parseInt(Timezone.getDateUTC(Date.parseExact(form.endDate + ( !!form.allDay ? " 00:00": " "+$.trim(form.endHour)), formatString ), tzId).getTime()) + (!!form.allDay ? 86400000 : 0) ,
				  allDay: ( !!form.allDay ? 1 : 0 ),
			      id: form.idEvent,
			      location: form.location,
			      category: form.category,
			      calendar: form.calendar,
			      summary: form.summary,
			      description: form.description,
			      timezone: tzId,
			      participants: $.map( form.attendee || [], function( attendee, i ){

						if( !attendee || attendee === "" )
							return( null );

						var participant = {};
						participant.user = (attendee!= User.me.id) ? DataLayer.get('participant', attendee).user : attendee ;
						
						if( participant.user === User.me.id ){
							return DataLayer.merge({
								isOrganizer: ( !i ? 1 : 0 ),
								alarms: participant.alarms = $.map( form.alarmTime || [], function( alarmTime, i ){
										if( alarmTime === "" )
											return( null );
										 return !!form.alarmId[i] ? { type: form.alarmType[i], unit: form.alarmUnit[i], time: form.alarmTime[i], id: form.alarmId[i] }:
																	{ type: form.alarmType[i],unit: form.alarmUnit[i], time: form.alarmTime[i] };
									}),
								status: form.status
							}, !!form.idEvent ? {id: attendee} : {user: participant.user});							
						}else{
							return {
								id: attendee,
								isOrganizer: ( !i ? 1 : 0 ),
								isExternal: !!parseInt(form.attendeeType[i]) ? 1 : 0
							};
						};
					  })
			    };
		    });

	return notArray ? res[0] : res;
    },

    encoder: function( evtObj ){
	
	if(!(!!evtObj))
		return undefined;

	var notArray = false;
      
	if( notArray = $.type(evtObj) !== "array" )
	    evtObj = [ evtObj ];

	var pref = User.preferences;

	var res =  DataLayer.encode('schedulable:preview', evtObj);

	if( !notArray ){
	    var dates = {};

	    for (var i=0; i < res.length; i++) {

		    var startDate = Date.parseExact( res[i]['startDate'], User.preferences.dateFormat );
		    var endDate   = Date.parseExact( res[i]['endDate'], User.preferences.dateFormat );

		    var counter   = startDate.clone();

		    while (counter.compareTo(startDate) == 0) {

			    if (!dates[counter.toString(User.preferences.dateFormat)]) 
					dates[counter.toString(User.preferences.dateFormat)] = { startDate:false, events:[] };
			    if (!dates[counter.toString(User.preferences.dateFormat)].startDate) 
					dates[counter.toString(User.preferences.dateFormat)].startDate = counter.toString(User.preferences.dateFormat);

			    dates[counter.toString(User.preferences.dateFormat)].events.push(res[i]);
			    counter.addDays(-1);
		    } 
	    }
	    res = { events_list: dates,
				count : res.length
			};
	}

	return notArray ? res[0] : res;
    }

});

DataLayer.codec( "participant", "detail", {
  
    encoder: function( participants ){

		var result = {};
		for( var i = 0; i< participants.length; i++ ){
			result[ participants[i].user.id || participants[i].user ] = participants[i];
		}
		return( result );
    }
});

DataLayer.codec( "calendar", "list", {
  
    encoder: function( calendars ){

		var result = { my_calendars: [], others_calendars: [] };

		for( var i = 0; i < calendars.length; i++ )
		{
			!!Calendar.signatureOf[ calendars[i].id ].isOwner ?
				result.my_calendars.push( calendars[i] ) :
				result.others_calendars.push( calendars[i] );
		}

		return { agendas_usuario: result };

    }

});

DataLayer.codec( "schedulable", "calendar", {

    decoder:function( evtObj ){
	
	if( notArray = $.type(evtObj) !== "array" )
	    evtObj = [ evtObj ];
	
	var res = $.map(evtObj, function( evt ){

		    return { id: evt.id,
			     summary: evt.title,
			     startTime: evt.start.getTime(),
			     endTime: ( !!evt.allDay ? (( evt.end || evt.start ).getTime() + 86400000) :( evt.end || evt.start ).getTime()),
			     allDay: ( !!evt.allDay ? 1 : 0 )
			     /*calendar: evt.id && User.calendarOf[ evt.id ].id*/ };
	});

	return notArray ? res[0] : res;
    },

    encoder: function( evtObj, filter ){
      
	if( !evtObj )
	    return( false );

//       if( notArray = $.type(evtObj) !== "array" )
// 	  evtObj = [ evtObj ];
// 	alert( evtObj );
	var filtered = evtObj;
	
	// repeat:{
// 	  frequency: "weekly",
// 	  endTime: Date.parse("+10 days").getTime(),
// 	  interval: 3,
// 	  byday: 3,
// 	  bysecond: 3,
// 	  byminute: 3,
// 	  bymonthday: 3,
// 	  byyearday: 3,
// 	  byweekno: 3,
// 	  bysetpos: 3,
// 	  wkst: "monday"/*,
// 	  duration: true*/
//       }

// 	$.each(evtObj, function( index, evt ){

// 		alert( dump(!!evt.repeat) );

// 		if( evt && !!evt.repeat && (evt.startTime < filter.end || (evt.repeat.endTime || 0) > filter.start ) )
// 		{
// // 		    var interval = evt.repeat.interval;
// 
// 		    var current = new Date( evt.startTime );
// 		    var begin = new Date( filter.start );
// 		    var end = new Date( filter.end );
// 		    var offset = evt.endTime - evt.startTime;
// 
// 		    var ocurrence = [];
// 
// 		    while( Date.compare( current, begin ) !== 1 )
// 			current = current.parse( "+" + evt.repeat.interval + " " + frequency + "s" );
// 
// 		    while( current.before( end ) && (ocurrence[ ocurrence.length ] = current) )
// 			current = current.parse( "+" + evt.repeat.interval + " " + frequency + "s" );
// 
// 		    for( var i = 0; i < ocurrences.length; i++ )
// 			filtered[ filtered.length ] = { URI: evt.URI,
// 							summary: evt.summary,
// 							start: new Date( ocurrences[i] ),
// 							end: new Date( ocurrences[i] + offset ),
// 							allDay: evt.allDay };
// 
// // 		    switch( evt.repeat.frequency )
// // 		    {
// // 		      case "yearly":   interval *= 12;
// // 		      case "monthly":  interval *= 30; //TODO: get the month variant
// // 		      case "weekly":   interval *= 7;
// // 		      case "daily":    interval *= 24;
// // 		      case "hourly":   interval *= 60;
// // 		      case "minutely": interval *= 60; 
// // 		      case "secondly": interval *= 1000;
// // 		    }
// // 		  
// // 		    var rest = ( filter.start - evt.startTime ) % interval;
// // 
// // 		    var offset = evt.endTime - evt.startTime;
// // 
// // 		    if( rest < offset )
// // 		    {
// // 			var freq = Math.floor( offset - rest / interval );
// // 
// // 			
// // 
// // 			for( var i = 0; i < freq; i++ )
// // 			{
// // 			     var ocurrence = filter.start + rest + ( interval * i );
// // 
// // 			     
// // 			}
// // 
// // 			
// // 		    }
// 		}

// 		if( parseInt( evt.endTime ) > filter[1][0][2] || parseInt( evt.startTime ) < filter[1][1][2] )
// 		    filtered[ filtered.length ] = evt;
// 
// 	});

	var grouped = {};

	$.map(filtered, function( evt ){

		evt.calendar = evt.calendar || "1";

		if( !grouped[ evt.calendar ] )
		    grouped[ evt.calendar ] = [];
			
		var calendar = DataLayer.get('calendar', evt.calendar);
		  
		grouped[ evt.calendar ].push( { id: evt.URI || evt.id,
						title: Encoder.htmlDecode(evt.summary),
						start: Timezone.getDateTimezone(new Date(parseInt( evt.startTime )), calendar.timezone, false),
						end:   Timezone.getDateTimezone(new Date(parseInt( evt.endTime ) - (!!parseInt(evt.allDay) ? 86400000 : 0)), calendar.timezone, false ),
						allDay: parseInt( evt.allDay ) } );
	});

	return(/* notArray ? filtered[0] :*/ grouped );
    },
    
    criteria: function( filter ){
      
		if( $.type(filter.start) !== 'date' )
			filter.start = new Date( filter.start * 1000 );
		if( $.type(filter.end) !== 'date' )
			filter.end = new Date( filter.end * 1000 );

		return { filter: ["AND", 

					[ ">=", "rangeStart", filter.start.toString( "dd-MM-yyyy" ) ],
					[ "<=", "rangeEnd", filter.end.toString( "dd-MM-yyyy" ) ],
					[ "IN", "calendar", Calendar.calendarIds ]

				  ], 
		 criteria: { deepness: 2 } }
    }
});


DataLayer.codec( "preference", "detail", {
  
	decoder:function( pref ){
	
		var res = [];

		pref.defaultAlarm = $.map( pref.alarmTime || [], function( alarmTime, i ){
		  
			return { type: pref.alarmType[i], 
				time: alarmTime,
				unit: pref.alarmUnit[i] };
			});

		$.each( pref, function( i, el ){

			res[ res.length ] = { name: i, value: el };

		});
		
		return( res );
      
	},

	encoder:function( pref ){
		return( pref );
	}

});

DataLayer.codec( "schedulable", "export", {
  
	decoder: function(){},
      
    encoder: function( signatures ){},
      
    criteria: function( filter ){
	
		if( isCal  = filter && filter.calendar )
			filter = filter.calendar;
			return {
				filter: filter ? [  "=", ( isCal ? "calendar" : "id" ), filter ] : false,
				criteria: { format: 'iCal', deepness: 2 }
			};
		}
});

Alarms.load();

// DataLayer.decoder( "participant", "detail", function( form ){
//  
// //     if( $.type( attObj ) !== "array" ){
// // 	notArray = true;
// // 	attObj = [ attObj ];
// //     }
// 
//     
// 
//     return( participants );
// 
// });

// DataLayer.decoder( "alarm", "detail", function( form ){
// 
// //       if( $.type( attObj ) !== "array" ){
// // 	    notArray = true;
// // 	    attObj = [ attObj ];
// //     }
// 
//     var alarms = [];
// 
//     if( form.alarmType /*&& !form.*/ )
// 	for( var i = 0; i < form.alarmType.length; i++ )
// 	{
// 	    if( form.alarmTime[i] === "" )
// 		continue;
// 	  
// 	    alarms[i] = { type: form.alarmType[i], 
// 			  unit: form.alarmUnit[i], 
// 			  time: form.alarmTime[i] };
// 	}
// 
//     return( alarms );
// 
// });

